#include "Weather.h"
#include "../SevenSegment.h"
#include "../Icons.h"
#include <Fonts/FreeSans9pt7b.h>
#include <Fonts/FreeSansBold9pt7b.h>
#include <Fonts/FreeSans12pt7b.h>
#include <Fonts/FreeSans18pt7b.h>
#include <HTTPClient.h>

RTC_DATA_ATTR uint64_t WatchFacePages::Weather::m_lastSyncTime = 0;
RTC_DATA_ATTR int WatchFacePages::Weather::m_lastCalculatedDay = 0XFFFFFFFF;
RTC_DATA_ATTR float WatchFacePages::Weather::m_locationLat = DEFAULT_LATITUDE;
RTC_DATA_ATTR float WatchFacePages::Weather::m_locationLon = DEFAULT_LONGITUDE;
RTC_DATA_ATTR char WatchFacePages::Weather::m_locationCity[128];

WatchFacePages::Weather::Weather(WatchyDisplay & display, WatchFeatures::WatchFeatures & features)
  : m_display(display), m_features(features)
{
}

void WatchFacePages::Weather::InitBoot()
{
  memcpy(m_locationCity, DEFAULT_CITY_NAME, sizeof(DEFAULT_CITY_NAME));
}

void WatchFacePages::Weather::InitWake()
{
}

void WatchFacePages::Weather::DrawPage(bool partialRefresh)
{
  if (m_lastSyncTime == 0) {
    Resync();
  }

  m_display.setFullWindow();
  m_display.fillScreen(GxEPD_WHITE);
  m_display.setTextColor(GxEPD_BLACK);

  if (m_lastSyncTime == 0) {
    m_display.setFont(&FreeSans12pt7b);
    int16_t x, y;
    uint16_t w, h;
    m_display.getTextBounds("Have not synced", 0, 0, &x, &y, &w, &h);
    m_display.setCursor(DISPLAY_WIDTH / 2 - w / 2, 110);
    m_display.print("Have not synced");
    m_display.display(partialRefresh);
    return;
  }

  bool weatherOutdated = false;

  if (m_features.rtc.GetTimestamp() - m_lastSyncTime > 86400) {
    weatherOutdated = true;
  }

  // City name and icon
  m_display.setFont(&FreeSans9pt7b);
  int16_t x, y;
  uint16_t w, h;
  
  m_display.drawBitmap(5, 5, Icons::city, 29, 23, GxEPD_BLACK);

  m_display.setCursor(39, 22);
  m_display.print(m_locationCity);
  // Separator
  m_display.fillRect(10, 35, DISPLAY_WIDTH - 20, 2, GxEPD_BLACK);

  // Temperature and weather icon
  // Separator 
  m_display.fillRect(10, 118, DISPLAY_WIDTH - 20, 2, GxEPD_BLACK);

  m_display.drawBitmap(0, 120, Icons::Weather::day_clear, 75, 75, GxEPD_BLACK);
  m_display.setFont(&FreeSans12pt7b);

  std::string temperature = "-20.5  C";

  m_display.getTextBounds(temperature.c_str(), 0, 0, &x, &y, &w, &h);
  m_display.setCursor(190 - w, 142);
  m_display.print(temperature.c_str());
  // Hacky degree symbol
  m_display.fillCircle(190 - 23, 129, 3, GxEPD_BLACK);
  m_display.fillCircle(190 - 23, 129, 1, GxEPD_WHITE);

  // Wind speed
  m_display.drawBitmap(165, 145, Icons::Weather::wind, 30, 30, GxEPD_BLACK);
  std::string windSpeed = "99 m/s";
  m_display.setFont(&FreeSans9pt7b);
  m_display.getTextBounds(windSpeed.c_str(), 0, 0, &x, &y, &w, &h);
  m_display.setCursor(160 - w, 165);
  m_display.print(windSpeed.c_str());

  // Humidity
  m_display.drawBitmap(165, 170, Icons::Weather::humidity, 30, 30, GxEPD_BLACK);
  std::string humidity = "100";
  m_display.getTextBounds(humidity.c_str(), 0, 0, &x, &y, &w, &h);
  m_display.setCursor(160 - w, 190);
  m_display.print(humidity.c_str());

  m_display.display(partialRefresh);
}

void WatchFacePages::Weather::Resync()
{
  uint64_t currentTime = m_features.rtc.GetTimestamp();

  if(m_lastSyncTime > 0 && currentTime - m_lastSyncTime < WEATHER_UPDATE_INTERVAL) {
    return;
  }

  if(!m_features.wifi.Connect()) {
    return;
  }

  if (DO_GEOLOCATION) {
    HTTPClient client;
    client.setConnectTimeout(3000); // 3 second max timeout

    client.begin("http://ip-api.com/line/");
    int httpCode = client.GET();
    if (httpCode != 200) {
      m_features.wifi.Disconnect();
      return;
    }

    String payload = client.getString();
    client.end();
    std::string reponse(payload.c_str());

    // Split into lines
    std::vector<std::string> lines;
    std::string::size_type pos = 0;
    std::string::size_type prev = 0;
    char delimiter = '\n';
    while ((pos = reponse.find(delimiter, prev)) != std::string::npos) {
      lines.push_back(reponse.substr(prev, pos - prev));
      prev = pos + 1;
    }

    // Check if we got enough lines
    if (lines.size() < 14) {
      m_features.wifi.Disconnect();
      return;
    }

    std::string location = lines[5];
    location = location.substr(0, 127);
    m_locationCity[location.length()] = '\0';
    memcpy(m_locationCity, location.c_str(), location.length());
    m_locationLat = std::stof(lines[7]);
    m_locationLon = std::stof(lines[8]);
  }
  
  m_features.wifi.Disconnect();

  m_lastSyncTime = currentTime;
}